/*!
 * \file    void.h
 * \brief   Template meta-programming void helpers
 *
 * Copyright (C) 2018 Christos Choutouridis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __utl_meta_void_h__
#define __utl_meta_void_h__

#include <utl/impl/version.h>
#include <utl/meta/bool.h>

/*! \ingroup meta
 * \defgroup void
 * void_ support header
 */
//! @{

namespace utl {

   /*!
    * Like boost::mpl we made void_ a complete type to allow it to be
    * instantiated so that it can be passed in as an object that can be
    * used to select an overloaded function.
    */
   struct void_ {
      typedef void_ type;
   };

   template<typename _Tp>
   struct is_void_
         : false_ { };

   template<>
   struct is_void_ <void_>
      : true_ { };

   template<typename _Tp>
   struct is_not_void_
      : true_ { };

   template<>
   struct is_not_void_<void_>
      : false_ { };

}

//!@}

#endif /* __utl_meta_void_h__ */
